/* arch/arm/mach-msm/kcj_reset_info.c
 *
 * Kyocera Reset Information
 *
 * This software is contributed or developed by KYOCERA Corporation.
 * (C) 2011 KYOCERA Corporation
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */
#include <linux/version.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/types.h>
#include <linux/time.h>
#include <linux/ctype.h>
#include <linux/console.h>
#include <linux/miscdevice.h>
#include <linux/fs.h>
#include <linux/ptrace.h>
#include <linux/spinlock.h>
#include <linux/init.h>
#include <asm/uaccess.h>
#include <asm/ioctl.h>
#include <asm/cputime.h>
#include <mach/msm_iomap.h>
#include <kcj_reset_info.h>

MODULE_LICENSE("GPL v2");

/****************************************************************************
 * Local Variable/Structure Declaration                                     
 ****************************************************************************/
spinlock_t           g_rstinfo_lock;
static t_RESET_INFO *l_rstinfo = (void *)(RSTINFO_START_ADDR);
static t_RESET_INFO  g_rstinfo_tmp;













static unsigned long rstinfo_calc_checksum
(
    int update
)
{
    unsigned long  sum;
    unsigned long  i;
    unsigned char *p;

    p = (unsigned char*)l_rstinfo;

    /* calc sum */
    for (i = 0, sum = 0; i<sizeof(t_RESET_INFO) - sizeof(unsigned long);i++, p++)
    {
        sum += *p;
    }

    if (update == false)
    {
        
        if (l_rstinfo->sum != sum)
        {
            printk(KERN_INFO "[OTA-DIAG]CHECKSUM ERROR\n");
            return false;
        }
        else
        {
            return true;
        }
    }
    else
    {
        
        l_rstinfo->sum = sum;
        return sum;
    }
}


 
 
 
 
 
 
void rstinfo_save
(
    struct pt_regs *regs
)
{
    struct  timespec  now;
    struct  timespec  uptime;
    struct  timespec  idletime;
            cputime_t cpu_idletime;

    
    if (regs == NULL)
    {
        printk(KERN_ERR "[RSTINFO]regs == NUL\n");
        return;
    }

    
    now          = current_kernel_time();
    cpu_idletime = cputime_add(init_task.utime, init_task.stime);
    do_posix_clock_monotonic_gettime(&uptime);
    monotonic_to_bootbased(&uptime);
    cputime_to_timespec(cpu_idletime, &idletime);

    
    spin_lock(&g_rstinfo_lock);

    
    memset(l_rstinfo, 0, sizeof(t_RESET_INFO));

    
    memcpy((void *)&(l_rstinfo->regs), (void *)regs, sizeof(struct pt_regs));

    
    memcpy(l_rstinfo->magic, "RSTINFO", RSTINFO_MAGIC_LEN);

    /* Time */
    l_rstinfo->crashtime.tv_sec  = now.tv_sec;
    l_rstinfo->crashtime.tv_nsec = now.tv_nsec;
    l_rstinfo->uptime.tv_sec     = uptime.tv_sec;
    l_rstinfo->uptime.tv_nsec    = uptime.tv_nsec;
    l_rstinfo->idletime.tv_sec   = idletime.tv_sec;
    l_rstinfo->idletime.tv_nsec  = idletime.tv_nsec;

    l_rstinfo->remaining = 0;

    
    if (user_mode(regs))
    {
        
        printk(KERN_DEBUG "[RSTINFO]MODE=USER\n");

        if (copy_from_user((void *)&(l_rstinfo->stacks), (void *)((regs->ARM_sp) &= ~0x03), sizeof(l_rstinfo->stacks)))
        {
            
            printk(KERN_ERR "[RSTINFO]Fail to SAVE RSTINFO[%d]", __LINE__);
            memset((void *)&(l_rstinfo->stacks), 0, sizeof(l_rstinfo->stacks));
        }
    }
    else
    {
        
        struct thread_info *thread;
        struct task_struct *tsk;
        unsigned long       stack_limit;
        unsigned long       dumpsize;

        printk(KERN_DEBUG "[RSTINFO]MODE=KERNEL\n");

        
        l_rstinfo->dump_begin = ((regs->ARM_sp) &= ~0x03) - 64;

        
        thread = current_thread_info();
        tsk = thread->task;
        stack_limit = (unsigned long)(thread + 1);
        l_rstinfo->stack_limit = stack_limit;

        dumpsize = THREAD_SIZE + (unsigned long)task_stack_page(tsk) - l_rstinfo->dump_begin;

        if (dumpsize > sizeof(l_rstinfo->stacks))
        {
            dumpsize = sizeof(l_rstinfo->stacks);
        }

        memcpy((void *)&(l_rstinfo->stacks), (void *)l_rstinfo->dump_begin, dumpsize);
    }

    
    rstinfo_calc_checksum(true);

    
    spin_unlock(&g_rstinfo_lock);

    return;
}
EXPORT_SYMBOL(rstinfo_save);


 
 
 
 
 
 
static void rstinfo_save_stack
(
    t_RESET_INFO *data
)
{
    struct  timespec  now;
    struct  timespec  uptime;
    struct  timespec  idletime;
            cputime_t cpu_idletime;

    
    if (data == NULL)
    {
        printk(KERN_ERR "[RSTINFO]data == NUL\n");
        return;
    }

    
    now          = current_kernel_time();
    cpu_idletime = cputime_add(init_task.utime, init_task.stime);
    do_posix_clock_monotonic_gettime(&uptime);
    monotonic_to_bootbased(&uptime);
    cputime_to_timespec(cpu_idletime, &idletime);

    
    spin_lock(&g_rstinfo_lock);

    
    memset(l_rstinfo, 0, sizeof(t_RESET_INFO));

    
    memcpy(l_rstinfo->magic, "RSTINFO", RSTINFO_MAGIC_LEN);

    
    l_rstinfo->crashtime.tv_sec  = now.tv_sec;
    l_rstinfo->crashtime.tv_nsec = now.tv_nsec;
    l_rstinfo->uptime.tv_sec     = uptime.tv_sec;
    l_rstinfo->uptime.tv_nsec    = uptime.tv_nsec;
    l_rstinfo->idletime.tv_sec   = idletime.tv_sec;
    l_rstinfo->idletime.tv_nsec  = idletime.tv_nsec;

    l_rstinfo->remaining = 0;
    memcpy(l_rstinfo->stacks, data->stacks, RSTINFO_STACK_SIZE);

    
    rstinfo_calc_checksum(true);

    
    spin_unlock(&g_rstinfo_lock);

    return;
}


 
 
 
 
 
 
 
 
 
 
 
 
 
int rstinfo_ioctl
(
    struct inode  *inode,
    struct file   *filp,
    unsigned int   cmd,
    unsigned long  arg
)
{
    switch (cmd)
    {
        case REQUEST_SAVE_STACK:

            printk(KERN_ERR "[RSTINFO]REQUEST_SAVE_STACK\n");

            
            if (copy_from_user((void *)&g_rstinfo_tmp, (void *)arg, sizeof(t_RESET_INFO)))
            {
                printk(KERN_ERR "[RSTINFO]Fail to SAVE RSTINFO[%d]", __LINE__);
                return -EFAULT;
            }

            
            rstinfo_save_stack(&g_rstinfo_tmp);
            break;

        case REQUEST_READ_MAGIC:

            printk(KERN_ERR "[RSTINFO]REQUEST_READ_MAGIC\n");

            
            if (strncmp(l_rstinfo->magic, "RSTINFO", RSTINFO_MAGIC_LEN))
            {
                printk(KERN_ERR "[RSTINFO]No Data[%d]", __LINE__);
                return -ENOENT;
            }
            break;

        default:

            printk(KERN_ERR "[RSTINFO]Cmd Error\n");

            return -EINVAL;
    }

    
    return 0;
}

static const struct file_operations t_reset_info = {
    .owner = THIS_MODULE,
    .ioctl = rstinfo_ioctl,
};

static struct miscdevice t_rstinfo_miscdev = {
    .minor = RSTINFO_MINOR,
    .name  = "rstinfo",
    .fops  = &t_reset_info,
};


 
 
 
 
 
 
 
static int __init rstinfo_init(void)
{
    int retval = 0;

    
    retval = misc_register(&t_rstinfo_miscdev);

    if (retval) {
        
        misc_deregister(&t_rstinfo_miscdev);
        return retval;
    }

    
    spin_lock_init(&g_rstinfo_lock);

    return retval;
}


 
 
 
 
 
 
static void __exit rstinfo_exit(void)
{
    
    misc_deregister(&t_rstinfo_miscdev);
}


module_init(rstinfo_init);
module_exit(rstinfo_exit);

