/*
 * This software is contributed or developed by KYOCERA Corporation.
 * (C) 2011 KYOCERA Corporation
*/
#include <linux/types.h>
#include <asm/uaccess.h>
#include <linux/kernel.h>
#include <asm/atomic.h>
#include <linux/i2c.h>
#include <linux/errno.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/input.h>
#include <linux/mutex.h>
#include <linux/miscdevice.h>
#include <linux/delay.h>
#include <linux/hrtimer.h>
#include <linux/irq.h>
#include <linux/gpio.h>
#include <mach/hs_io_ctl_a.h>
#include <gp2ap002a00f.h>
#include <mach/proc_comm_kyocera.h>


#define PROXIMITY_DEV_NAME "GP2AP002A00F"

#define LIGHT_SENSOR_POLL_TIME_MSEC (200)

#define LIGHT_SENSOR_LUX_VALUE_0        (0)
#define LIGHT_SENSOR_LUX_VALUE_0_MIN    (0)
#define LIGHT_SENSOR_LUX_VALUE_0_MAX    (36)
#define LIGHT_SENSOR_LUX_VALUE_1        (100)
#define LIGHT_SENSOR_LUX_VALUE_1_MIN    (37)
#define LIGHT_SENSOR_LUX_VALUE_1_MAX    (68)
#define LIGHT_SENSOR_LUX_VALUE_2        (400)
#define LIGHT_SENSOR_LUX_VALUE_2_MIN    (69)
#define LIGHT_SENSOR_LUX_VALUE_2_MAX    (86)
#define LIGHT_SENSOR_LUX_VALUE_3        (900)
#define LIGHT_SENSOR_LUX_VALUE_3_MIN    (87)
#define LIGHT_SENSOR_LUX_VALUE_3_MAX    (93)
#define LIGHT_SENSOR_LUX_VALUE_4        (1200)
#define LIGHT_SENSOR_LUX_VALUE_4_MIN    (94)
#define LIGHT_SENSOR_LUX_VALUE_4_MAX    (166)
#define LIGHT_SENSOR_LUX_VALUE_5        (30000)
#define LIGHT_SENSOR_LUX_VALUE_5_MIN    (167)
#define LIGHT_SENSOR_LUX_VALUE_5_MAX    (255)
#define LIGHT_SENSOR_LUX_VALUE_ERR      (0)

#define LIGHT_SENSOR_LUX_VALUE_LV1        (0)
#define LIGHT_SENSOR_LUX_VALUE_LV1_MIN    (0)
#define LIGHT_SENSOR_LUX_VALUE_LV1_MAX    (115)
#define LIGHT_SENSOR_LUX_VALUE_LV2        (100)
#define LIGHT_SENSOR_LUX_VALUE_LV2_MIN    (116)
#define LIGHT_SENSOR_LUX_VALUE_LV2_MAX    (168)
#define LIGHT_SENSOR_LUX_VALUE_LV3        (400)
#define LIGHT_SENSOR_LUX_VALUE_LV3_MIN    (169)
#define LIGHT_SENSOR_LUX_VALUE_LV3_MAX    (198)
#define LIGHT_SENSOR_LUX_VALUE_LV4        (900)
#define LIGHT_SENSOR_LUX_VALUE_LV4_MIN    (50)
#define LIGHT_SENSOR_LUX_VALUE_LV4_MAX    (53)
#define LIGHT_SENSOR_LUX_VALUE_LV5        (1200)
#define LIGHT_SENSOR_LUX_VALUE_LV5_MIN    (54)
#define LIGHT_SENSOR_LUX_VALUE_LV5_MAX    (83)
#define LIGHT_SENSOR_LUX_VALUE_LV6        (30000)
#define LIGHT_SENSOR_LUX_VALUE_LV6_MIN    (84)
#define LIGHT_SENSOR_LUX_VALUE_ERR        (0)

#define RTN_OK    (0)
#define RTN_NG    (-1)

#define PROXIMITY_ON    (0)
#define PROXIMITY_OFF   (1)

static struct gp2_reg_set reg_set_list[GP2_MODE_MAX_NUM] = {
	{ 
		{0x00}, 			
		{0x08}, 			
		{0x80,0x40,0x02}, 	
		{0x00,0x04},		
		{0x00,0x01,0x00},	
		{0x00}, 			
	},
	{ 
		{0x00}, 			
		{0x08}, 			
		{0x00,0x40,0x00}, 	
		{0x00,0x04},		
		{0x00,0x01,0x02},	
		{0x00}, 			
	},
	{ 
		{0x00}, 			
		{0x08}, 			
		{0x00,0x02,0x00}, 	
		{0x00,0x04},		
		{0x00,0x01,0x02},	
		{0x00}, 			
	},
	{ 
		{0x00}, 			
		{0x08}, 			
		{0x80,0x40,0x02}, 	
		{0x00,0x04},		
		{0x00,0x01,0x00},	
		{0x00}, 			
	},
};


const int IRQ_NUM = MSM_GPIO_TO_INT(GPIO_PS_OUT);
int IRQ_LIST_NO = 0;
void *irq_dev_id = (void *)&IRQ_NUM;



static int gpio_photo_on_value = GPIO_LO;





static int gp2_mode = GP2_MODE_B1;
static int gp2_mode_chg_req = GP2_MODE_B1;



static int debug_log_enable = 0;



#ifdef SENSOR_DPRINTK
#undef SENSOR_DPRINTK
#endif
#define SENSOR_DPRINTK(msg,...) \
    if(debug_log_enable) printk("[GP2AP002A00F]:%d " msg,__LINE__,##__VA_ARGS__)



static struct i2c_client *this_client = NULL;
struct delayed_work gp2_work;
struct input_dev *input;

struct delayed_work gp2_light_work;
struct input_dev *input2;



struct proximity_data {
    atomic_t enable;                
    atomic_t delay;                 

    int last_data;

};
static struct proximity_data proximity_data_tbl;

struct light_data {
    atomic_t enable;                
    atomic_t delay;                 
    int last_data;
};
static struct light_data light_data_tbl;



static int req_irq_flg = 0;




#include <linux/wakelock.h>
static struct wake_lock proximity_wake_lock;






static void GP2AP002A00F_poll_MODE_A(struct work_struct *work);
static void GP2AP002A00F_poll_MODE_B(struct work_struct *work);
static irqreturn_t GP2AP002A00F_isr(int irq, void* dev_id);
static int GP2AP002A00F_init_mode(void);
static int GP2AP002A00F_init_input(void);





static void GP2AP002A00F_poweron_setting(void)
{
    u8 wbuf[2] = {0};

    SENSOR_DPRINTK("%s START.\n",__func__);
    HS_A_PLOXIMITY_ON_HI();
    mdelay(1);

    gpio_tlmm_config(GPIO_CFG(GPIO_PS_OUT, 0, GPIO_CFG_INPUT, GPIO_CFG_PULL_UP, GPIO_CFG_2MA), GPIO_CFG_ENABLE);

    wbuf[0] = 0x04;
    wbuf[1] = 0x00;
    i2c_master_send(this_client, wbuf, 2);

    wbuf[0] = 0x01;
    wbuf[1] = 0x08;
    i2c_master_send(this_client, wbuf, 2);

    wbuf[0] = 0x02;
    wbuf[1] = 0x40;
    i2c_master_send(this_client, wbuf, 2);

    wbuf[0] = 0x03;
    wbuf[1] = 0x04;
    i2c_master_send(this_client, wbuf, 2);

    wbuf[0] = 0x06;
    wbuf[1] = 0x10;
    i2c_master_send(this_client, wbuf, 2);

    gpio_direction_output( GPIO_PS_OUT, GPIO_LO );
    gpio_tlmm_config(GPIO_CFG(GPIO_PS_OUT, 0, GPIO_CFG_INPUT, GPIO_CFG_PULL_DOWN, GPIO_CFG_2MA), GPIO_CFG_ENABLE);
    SENSOR_DPRINTK("%s END.\n",__func__);
}

static void GP2AP002A00F_poweroff_setting(void)
{
    SENSOR_DPRINTK("%s START.\n",__func__);
    HS_A_PLOXIMITY_ON_LO();
    SENSOR_DPRINTK("%s END.\n",__func__);
}

static void proximity_enable_setting(void)
{
    u8 wbuf[2] = {0};
    int ret = 0;
    
    SENSOR_DPRINTK("%s START.\n",__func__);
    gpio_tlmm_config(GPIO_CFG(GPIO_PS_OUT, 0, GPIO_CFG_INPUT, GPIO_CFG_PULL_UP, GPIO_CFG_2MA), GPIO_CFG_ENABLE);


    wbuf[0] = 0x01;
    wbuf[1] = 0x08;
    i2c_master_send(this_client, wbuf, 2);

    wbuf[0] = 0x02;
    wbuf[1] = 0x40;
    i2c_master_send(this_client, wbuf, 2);

    wbuf[0] = 0x03;
    wbuf[1] = 0x04;
    i2c_master_send(this_client, wbuf, 2);

    wbuf[0] = 0x04;
    wbuf[1] = 0x03;
    i2c_master_send(this_client, wbuf, 2);
    
	wbuf[0] = 0x06;
	wbuf[1] = 0x18;
	i2c_master_send(this_client, wbuf, 2);

    ret = request_irq(IRQ_NUM,GP2AP002A00F_isr,IRQF_TRIGGER_LOW,PROXIMITY_DEV_NAME,irq_dev_id);
    req_irq_flg = 1;
    enable_irq_wake(IRQ_NUM);

	wbuf[0] = 0x86;
	wbuf[1] = 0x00;
	i2c_master_send(this_client, wbuf, 2);

    SENSOR_DPRINTK("%s END.\n",__func__);
}
static void proximity_disable_setting(void)
{
    u8 wbuf[2] = {0};

    SENSOR_DPRINTK("%s START.\n",__func__);
    atomic_set(&proximity_data_tbl.enable, 0);
    cancel_delayed_work_sync(&gp2_work);

    if( req_irq_flg ) {
        free_irq(IRQ_NUM,irq_dev_id);
        req_irq_flg = 0;
        SENSOR_DPRINTK("%s free_irq\n",__func__);
    }
    disable_irq_wake(IRQ_NUM); 

    wbuf[0] = 0x04;
    wbuf[1] = 0x00;
    i2c_master_send(this_client, wbuf, 2);

    wbuf[0] = 0x01;
    wbuf[1] = 0x08;
    i2c_master_send(this_client, wbuf, 2);

    wbuf[0] = 0x02;
    wbuf[1] = 0x40;
    i2c_master_send(this_client, wbuf, 2);

    wbuf[0] = 0x03;
    wbuf[1] = 0x04;
    i2c_master_send(this_client, wbuf, 2);

    wbuf[0] = 0x06;
    wbuf[1] = 0x10;
    i2c_master_send(this_client, wbuf, 2);

    gpio_tlmm_config(GPIO_CFG(GPIO_PS_OUT, 0, GPIO_CFG_INPUT, GPIO_CFG_PULL_DOWN, GPIO_CFG_2MA), GPIO_CFG_ENABLE);
    SENSOR_DPRINTK("%s END.\n",__func__);
}






static void
GP2AP002A00F_poll_MODE_A(struct work_struct *work)
{
	u8 rbuf[2] = {0};

	i2c_smbus_read_i2c_block_data(this_client, 0x00, 2, rbuf);


    if (rbuf[1] & 0x01)
    {

        proximity_data_tbl.last_data = PROXIMITY_ON * 1000;
        SENSOR_DPRINTK("DEBUG:GP2AP002A00F_poll_MODE_A get_data:%d\n",PROXIMITY_ON * 1000);

        input_report_abs(input, ABS_X, PROXIMITY_ON*1000);
    }
    else
    {

        proximity_data_tbl.last_data = PROXIMITY_OFF * 1000;
        SENSOR_DPRINTK("DEBUG:GP2AP002A00F_poll_MODE_A get_data:%d\n",PROXIMITY_OFF * 1000);

        input_report_abs(input, ABS_X, PROXIMITY_OFF*1000);
    }

    input_sync(input);

	if(gp2_mode != gp2_mode_chg_req)
	{
		gp2_mode = gp2_mode_chg_req;
		GP2AP002A00F_init_mode();
	}
	else
	{
		schedule_delayed_work(&gp2_work, msecs_to_jiffies(10));
	}
}




static void
GP2AP002A00F_poll_MODE_B(struct work_struct *work)
{
	u8 rbuf[2] = {0};
	u8 wbuf[2];
	u8 Vout = gpio_get_value( GPIO_PS_OUT );
	
	if(Vout == 1)
	{
		return;
	}

	i2c_smbus_read_i2c_block_data(this_client, 0x00, 2, rbuf);


    if (rbuf[1] & 0x01)
    {

        proximity_data_tbl.last_data = PROXIMITY_ON * 1000;
        SENSOR_DPRINTK("DEBUG:GP2AP002A00F_poll_MODE_B get_data:%d\n",PROXIMITY_ON * 1000);

        input_report_abs(input, ABS_X, PROXIMITY_ON*1000); 
    }
    else
    {

        proximity_data_tbl.last_data = PROXIMITY_OFF * 1000;
        SENSOR_DPRINTK("DEBUG:GP2AP002A00F_poll_MODE_B get_data:%d\n",PROXIMITY_OFF * 1000);

        input_report_abs(input, ABS_X, PROXIMITY_OFF*1000); 

        wake_lock_timeout(&proximity_wake_lock, 2 * HZ);  
    }

    input_sync(input);


	wbuf[0] = 0x02;
	if((rbuf[1]&0x01) == 0)
	{
		
		wbuf[1] = 0x00;
		
		wbuf[1] |= 0x40;
		

        wbuf[1] |= 0x0A;

		i2c_master_send(this_client, wbuf, 2);
	}
	else
	{
		
		wbuf[1] = 0x00;
		
		wbuf[1] |= 0x20;
		
		wbuf[1] |= 0x0A;
		i2c_master_send(this_client, wbuf, 2);
	}


	wbuf[0] = 0x06;
	wbuf[1] = 0x18;
	i2c_master_send(this_client, wbuf, 2);

	
	if(gp2_mode != gp2_mode_chg_req)
	{

		if (req_irq_flg) {
		    free_irq(IRQ_NUM,irq_dev_id);
		    req_irq_flg = 0;
		}

		gp2_mode = gp2_mode_chg_req;
		GP2AP002A00F_init_mode();
	}
	else
	{
		
		wbuf[0] = 0x86;
		wbuf[1] = 0x00;
		i2c_master_send(this_client, wbuf, 2);

		enable_irq(IRQ_NUM);

	}
}


static irqreturn_t
GP2AP002A00F_isr(int irq, void* dev_id)
{
	if(irq_dev_id != dev_id)
	{
        
        SENSOR_DPRINTK("DEBUG:GP2AP002A00F_isr not mach device id\n");
        
		return IRQ_NONE;
	}
	if(gpio_get_value( GPIO_PS_OUT ) != 0)
	{
        
        SENSOR_DPRINTK("DEBUG:not interrupt from GPIO[106]\n");
        
		return IRQ_NONE;
	}
	

	disable_irq_nosync(irq);


	INIT_DELAYED_WORK(&gp2_work,GP2AP002A00F_poll_MODE_B);
	schedule_delayed_work(&gp2_work, msecs_to_jiffies(0));
	
	return IRQ_HANDLED;
}


static int
GP2AP002A00F_set_shutdown_mode(void)
{
	u8 wbuf[2];


    
    wbuf[0] = 0x86;
    wbuf[1] = 0x18;
    i2c_master_send(this_client, wbuf, 2);


    if( req_irq_flg ) {
        free_irq(IRQ_NUM,irq_dev_id);
        req_irq_flg = 0;
    }



	
	wbuf[0] = 0x04;
	wbuf[1] = 0;

	i2c_master_send(this_client, wbuf, 2);
	
	return 0;
}

static int
GP2AP002A00F_init_mode(void)
{
    u8 wbuf[2];
    int ret = 0;

    printk("[GP2AP002A00F] START: GP2AP002A00F_init_mode\n");

    
    GP2AP002A00F_set_shutdown_mode();

    
    wbuf[0] = 0x86;
    wbuf[1] = 0x18;
    i2c_master_send(this_client, wbuf, 2);


	
	wbuf[0] = 0x01;
	
	wbuf[1] = reg_set_list[gp2_mode].gain.led;
	i2c_master_send(this_client, wbuf, 2);

    printk("[GP2AP002A00F] DEBUG:init_mode GAIN:0x%x\n",wbuf[1]);

	
	
	
	wbuf[0] = 0x02;
	
	wbuf[1] = reg_set_list[gp2_mode].hys.hysd;
	
	wbuf[1] |= reg_set_list[gp2_mode].hys.hysc;
	
	wbuf[1] |= reg_set_list[gp2_mode].hys.hysf;
	i2c_master_send(this_client, wbuf, 2);

    printk("[GP2AP002A00F] DEBUG:init_mode HYS:0x%x\n",wbuf[1]);


	
	wbuf[0] = 0x03;
	
	wbuf[1] = reg_set_list[gp2_mode].cycle.cycl;
	
	wbuf[1] |= reg_set_list[gp2_mode].cycle.osc;
	i2c_master_send(this_client, wbuf, 2);

    printk("[GP2AP002A00F] DEBUG:init_mode CYCLE:0x%x\n",wbuf[1]);


	
	wbuf[0] = 0x04;
	
	wbuf[1] = reg_set_list[gp2_mode].opmod.ssd;
	
	wbuf[1] |= reg_set_list[gp2_mode].opmod.vcon;

    printk("[GP2AP002A00F] DEBUG:init_mode OPMOD:0x%x\n",wbuf[1]);


	i2c_master_send(this_client, wbuf, 2);

	
	wbuf[0] = 0x86;
	wbuf[1] = 0x00;
	i2c_master_send(this_client, wbuf, 2);

	if(reg_set_list[gp2_mode].opmod.vcon == 0x00)
	{
		INIT_DELAYED_WORK(&gp2_work,GP2AP002A00F_poll_MODE_A);
		schedule_delayed_work(&gp2_work, msecs_to_jiffies(1000));
	}
	else
	{
		
		

		ret = request_irq(IRQ_NUM,GP2AP002A00F_isr,IRQF_TRIGGER_LOW,PROXIMITY_DEV_NAME,irq_dev_id);
		req_irq_flg = 1;
		enable_irq_wake(IRQ_NUM);

		if(ret)
		{
            
            printk("[GP2AP002A00F] ERR:request_irq() failed [%d]\n",ret);
            
		}
	}


    printk("[GP2AP002A00F] END: GP2AP002A00F_init_mode\n");

	return 0;
}

int GP2AP002A00F_light_change_lux( unsigned int *data )
{

    int i_Ret = RTN_OK;

    
    if ( *data >= LIGHT_SENSOR_LUX_VALUE_0_MIN && 
         *data <= LIGHT_SENSOR_LUX_VALUE_0_MAX )
    {
        *data = LIGHT_SENSOR_LUX_VALUE_0;
    }
    else if ( *data >= LIGHT_SENSOR_LUX_VALUE_1_MIN && 
              *data <= LIGHT_SENSOR_LUX_VALUE_1_MAX )
    {
        *data = LIGHT_SENSOR_LUX_VALUE_1;
    }
    else if ( *data >= LIGHT_SENSOR_LUX_VALUE_2_MIN && 
              *data <= LIGHT_SENSOR_LUX_VALUE_2_MAX )
    {
        *data = LIGHT_SENSOR_LUX_VALUE_2;
    }
    else if ( *data >= LIGHT_SENSOR_LUX_VALUE_3_MIN && 
              *data <= LIGHT_SENSOR_LUX_VALUE_3_MAX )
    {
        *data = LIGHT_SENSOR_LUX_VALUE_3;
    }
    else if ( *data >= LIGHT_SENSOR_LUX_VALUE_4_MIN && 
              *data <= LIGHT_SENSOR_LUX_VALUE_4_MAX )
    {
        *data = LIGHT_SENSOR_LUX_VALUE_4;
    }
    else if ( *data >= LIGHT_SENSOR_LUX_VALUE_5_MIN && 
              *data <= LIGHT_SENSOR_LUX_VALUE_5_MAX )
    {
        *data = LIGHT_SENSOR_LUX_VALUE_5;
    }
    else
    {
        
        printk("GP2AP002A00F_light_change_lux_ERROR value : %d\n", *data);
        *data = LIGHT_SENSOR_LUX_VALUE_ERR;
        i_Ret = RTN_NG;
    }

    return( i_Ret );
}

int GP2AP002A00F_light_change_lux_Lo( unsigned int *data )
{
    
    if ( *data <= LIGHT_SENSOR_LUX_VALUE_LV1_MAX ) {
        *data = LIGHT_SENSOR_LUX_VALUE_LV1;
    }
    else if ( *data >= LIGHT_SENSOR_LUX_VALUE_LV2_MIN && 
              *data <= LIGHT_SENSOR_LUX_VALUE_LV2_MAX ) {
        *data = LIGHT_SENSOR_LUX_VALUE_LV2;
    }
    else {
        *data = LIGHT_SENSOR_LUX_VALUE_LV3;
    }
    
    return RTN_OK;
}

int GP2AP002A00F_light_change_lux_Hi( unsigned int *data )
{
    
    if ( *data <= LIGHT_SENSOR_LUX_VALUE_LV4_MAX ) {
        *data = LIGHT_SENSOR_LUX_VALUE_LV4;
    }
    else if ( *data >= LIGHT_SENSOR_LUX_VALUE_LV5_MIN && 
              *data <= LIGHT_SENSOR_LUX_VALUE_LV5_MAX ) {
        *data = LIGHT_SENSOR_LUX_VALUE_LV5;
    }
    else {
        *data = LIGHT_SENSOR_LUX_VALUE_LV6;
    }
    
    return RTN_OK;
}

int GP2AP002A00F_light_change_lux_LoHi( unsigned int *data )
{

    int i_Ret = RTN_OK;

    
    switch(gpio_photo_on_value) {
        case GPIO_LO:    
            
            SENSOR_DPRINTK
                ("DEBUG:GP2AP002A00F_light_change_lux_LoHi GPIO_LO data:%d\n",*data);
            
            
            if(*data > LIGHT_SENSOR_LUX_VALUE_LV3_MAX) {
                
                
                HS_A_PHOTO_ON_HI();
                
                gpio_photo_on_value = GPIO_HI;
            }

            
            GP2AP002A00F_light_change_lux_Lo(data);
            break;

        case GPIO_HI:    
            
            SENSOR_DPRINTK
                ("DEBUG:GP2AP002A00F_light_change_lux_LoHi GPIO_HI data:%d\n",*data);
            
            
            if(*data < LIGHT_SENSOR_LUX_VALUE_LV4_MIN) {
                
                
                HS_A_PHOTO_ON_LO();
                
                gpio_photo_on_value = GPIO_LO;
            }

            
            GP2AP002A00F_light_change_lux_Hi(data);
            break;

        default:
            printk("[GP2AP002A00F] ERROR:light_change_lux GPIO_PHOTO_ON_value : %d\n", gpio_photo_on_value);
            *data = LIGHT_SENSOR_LUX_VALUE_ERR;
            i_Ret = RTN_NG;
            break;
    }

    return( i_Ret );
}


#define LIGHT_SENSOR_CHECK_WAITTIME     (10000)
int GP2AP002A00F_wait_lux_chg( unsigned int data )
{
    int ret = 0;
    static unsigned int wait_lux_chg = 0;
    
    if(data != LIGHT_SENSOR_LUX_VALUE_LV1)
    {
        wait_lux_chg = 0;
    }

    if (LIGHT_SENSOR_CHECK_WAITTIME < wait_lux_chg) {
        wait_lux_chg = 0;
    }

    if (0 == wait_lux_chg) {
        ret = 1;
    }

    wait_lux_chg += LIGHT_SENSOR_POLL_TIME_MSEC;

SENSOR_DPRINTK("DEBUG:GP2AP002A00F_wait_lux_chg wait_lux_chg=%d ret=%d\n",wait_lux_chg,ret);
    return ret;
}




static void
GP2AP002A00F_poll_light_sensor(struct work_struct *work)
{
    unsigned int data = 0;
    int i_Ret = RTN_OK;

    proc_comm_rpc_apps_to_modem(PROC_COMM_SUB_CMD_GET_L_SENSOR_V, &data);

    SENSOR_DPRINTK("DEBUG:GP2AP002A00F_poll_light_sensor get_data:%d\n",data);



#if !defined(CONFIG_TARGET_PROTOTYPE_WS2) & !defined(CONFIG_TARGET_PROTOTYPE_WS3)
    
    
    i_Ret = GP2AP002A00F_light_change_lux( &data );
#else 
    i_Ret = GP2AP002A00F_light_change_lux_LoHi( &data );
#endif 


    if ( RTN_NG == i_Ret )
    {
        

        printk("[GP2AP002A00F] ERR:GP2AP002A00F_light_change_lux_error\n");

    }
    


    
    light_data_tbl.last_data = data * 1000;
    


    if(GP2AP002A00F_wait_lux_chg(data))
    {
      input_report_abs(input2, ABS_X, data*1000);
      input_sync(input2);

      SENSOR_DPRINTK("DEBUG:GP2AP002A00F_poll_light_sensor input_report_abs_data:%d\n",data * 1000);

    }

    schedule_delayed_work(&gp2_light_work, msecs_to_jiffies(LIGHT_SENSOR_POLL_TIME_MSEC));
}





static int
proximity_enable(void)
{

    SENSOR_DPRINTK("START:proximity_enable:%d\n",atomic_read(&proximity_data_tbl.enable));

    if (!atomic_cmpxchg(&proximity_data_tbl.enable, 0, 1)) {
        
        
        
        proximity_data_tbl.last_data = PROXIMITY_OFF * 1000;
        input_report_abs(input, ABS_X, proximity_data_tbl.last_data);
        input_sync(input);
        
        
        if (!atomic_read(&light_data_tbl.enable))
        {
          GP2AP002A00F_poweron_setting();
        }
        proximity_enable_setting();
        atomic_set(&proximity_data_tbl.enable, 1);
        
    }


    SENSOR_DPRINTK("END:proximity_enable:%d\n",atomic_read(&proximity_data_tbl.enable));

    return 0;
}

static int
proximity_disable(void)
{

    SENSOR_DPRINTK("START:proximity_disable proximity_data_tbl.enable:%d light_data_tbl.enable:%d\n",
                    atomic_read(&proximity_data_tbl.enable),atomic_read(&light_data_tbl.enable));

    if (atomic_cmpxchg(&proximity_data_tbl.enable, 1, 0)) {
        
        proximity_disable_setting();
        if (!atomic_read(&light_data_tbl.enable)) {
            GP2AP002A00F_poweroff_setting();            
        }
        
    }

    SENSOR_DPRINTK("END:proximity_disable proximity_data_tbl.enable:%d light_data_tbl.enable:%d\n",
                    atomic_read(&proximity_data_tbl.enable),atomic_read(&light_data_tbl.enable));

    return 0;
}

static ssize_t
proximity_enable_show(struct device *dev,
        struct device_attribute *attr,
        char *buf)
{
    return sprintf(buf, "%d\n", atomic_read(&proximity_data_tbl.enable));
}

static ssize_t
proximity_enable_store(struct device *dev,
        struct device_attribute *attr,
        const char *buf,
        size_t count)
{
    int value;

    value = simple_strtoul(buf, NULL, 10);

    SENSOR_DPRINTK("DEBUG:proximity_enable_store value:%d\n",value);

    if (value != 0 && value != 1) {

    printk("[GP2AP002A00F] ERR:proximity_enable_store error_value:%d\n",value);

        return count;
    }

    if (value) {
        proximity_enable();
    }
    else {
        proximity_disable();
    }

    return count;
}

static ssize_t
proximity_delay_show(struct device *dev,
        struct device_attribute *attr,
        char *buf)
{
    return sprintf(buf, "%d\n", atomic_read(&proximity_data_tbl.delay));
}

static ssize_t
proximity_delay_store(struct device *dev,
        struct device_attribute *attr,
        const char *buf,
        size_t count)
{
    int value;

    value = simple_strtoul(buf, NULL, 10);

    atomic_set(&proximity_data_tbl.delay, value);

    return count;
}

static ssize_t
proximity_wake_store(struct device *dev,
                                struct device_attribute *attr,
                                const char *buf, size_t count)
{
    struct input_dev *input_dat = to_input_dev(dev);
    static atomic_t serial = ATOMIC_INIT(0);

    input_report_abs(input_dat, ABS_MISC, atomic_inc_return(&serial));

    return count;
}

static ssize_t
proximity_data_show(struct device *dev,
                               struct device_attribute *attr, char *buf)
{
    return sprintf(buf, "%d\n", proximity_data_tbl.last_data);
}


static ssize_t GP2AP002A00F_dbglog_show(struct device *dev,
                               struct device_attribute *attr, char *buf)
{
    debug_log_enable ^= 1;

    if(debug_log_enable){
        printk("[GP2AP002A00F] DEBUG: DEBUG_ON\n");
    }
    else{
        printk("[GP2AP002A00F] DEBUG: DEBUG_OFF\n");
    }

    return sprintf(buf, "%d\n", debug_log_enable);
}



static DEVICE_ATTR(enable, S_IRUGO|S_IWUSR|S_IWGRP,
                    proximity_enable_show, proximity_enable_store);
static DEVICE_ATTR(delay, S_IRUGO|S_IWUSR|S_IWGRP,
                    proximity_delay_show, proximity_delay_store);
static DEVICE_ATTR(wake, S_IWUSR|S_IWGRP, NULL, proximity_wake_store);
static DEVICE_ATTR(data, S_IRUGO, proximity_data_show, NULL);


static DEVICE_ATTR(dbglog, S_IRUGO, GP2AP002A00F_dbglog_show, NULL);


static struct attribute *proximity_attributes[] = {
    &dev_attr_enable.attr,
    &dev_attr_delay.attr,
    &dev_attr_wake.attr,
    &dev_attr_data.attr,

    &dev_attr_dbglog.attr,

    NULL
};

static struct attribute_group proximity_attribute_group = {
    .attrs = proximity_attributes
};

static int
light_enable(void)
{

    SENSOR_DPRINTK("START:light_enable:%d\n",atomic_read(&light_data_tbl.enable));


    if (!atomic_cmpxchg(&light_data_tbl.enable, 0, 1)) {
        
        
        if (!atomic_read(&proximity_data_tbl.enable))
        {
          GP2AP002A00F_poweron_setting();
        }
        
       
        atomic_set(&light_data_tbl.enable, 1);
        schedule_delayed_work(&gp2_light_work, 0); 
    }


    SENSOR_DPRINTK("END:light_enable:%d\n",atomic_read(&light_data_tbl.enable));

    return 0;
}

static int
light_disable(void)
{

    SENSOR_DPRINTK("START:light_disable light_data_tbl.enable:%d proximity_data_tbl.enable:%d\n",
                    atomic_read(&light_data_tbl.enable),atomic_read(&proximity_data_tbl.enable));

    if (atomic_cmpxchg(&light_data_tbl.enable, 1, 0)) {

        if (!atomic_read(&proximity_data_tbl.enable)) {
            
            
            GP2AP002A00F_poweroff_setting();
            
        }

        atomic_set(&light_data_tbl.enable, 0);

        
        
        HS_A_PHOTO_ON_LO();
        
        
        gpio_photo_on_value = GPIO_LO;

        cancel_delayed_work_sync(&gp2_light_work);

        input_report_abs(input2, ABS_X, light_data_tbl.last_data);
        input_sync(input2);

    }

    SENSOR_DPRINTK("END:light_disable light_data_tbl.enable:%d proximity_data_tbl.enable:%d\n",
                    atomic_read(&light_data_tbl.enable),atomic_read(&proximity_data_tbl.enable));

    return 0;
}

ssize_t
light_enable_show(struct device *dev,
        struct device_attribute *attr,
        char *buf)
{
    return sprintf(buf, "%d\n", atomic_read(&light_data_tbl.enable));
}

ssize_t
light_enable_store(struct device *dev,
        struct device_attribute *attr,
        const char *buf,
        size_t count)
{
    int value;

    value = simple_strtoul(buf, NULL, 10);

    SENSOR_DPRINTK("DEBUG:light_enable_store value:%d\n",value);


    if (value != 0 && value != 1) {

    printk("[GP2AP002A00F] ERR:light_enable_store error_value:%d\n",value);

        return count;
    }

    if (value) {
        light_enable();
    }
    else {
        light_disable();
    }

    return count;
}

ssize_t
light_delay_show(struct device *dev,
        struct device_attribute *attr,
        char *buf)
{
    return sprintf(buf, "%d\n", atomic_read(&light_data_tbl.delay));
}

ssize_t
light_delay_store(struct device *dev,
        struct device_attribute *attr,
        const char *buf,
        size_t count)
{
    int value;

    value = simple_strtoul(buf, NULL, 10);

    atomic_set(&light_data_tbl.delay, value);

    return count;
}

ssize_t
light_wake_store(struct device *dev,
                                struct device_attribute *attr,
                                const char *buf, size_t count)
{
    struct input_dev *input_dat = to_input_dev(dev);
    static atomic_t serial = ATOMIC_INIT(0);

    input_report_abs(input_dat, ABS_MISC, atomic_inc_return(&serial));

    return count;
}

ssize_t
light_data_show(struct device *dev,
                               struct device_attribute *attr, char *buf)
{
    return sprintf(buf, "%d\n", light_data_tbl.last_data);
}

extern int light_create_group(struct input_dev *input);


static int
GP2AP002A00F_init_input(void)
{
	int err;


    printk("[GP2AP002A00F] START:GP2AP002A00F_init_input\n");


    input = input_allocate_device();
    input->name = "proximity";
    input->id.bustype = BUS_I2C;

    input_set_capability(input, EV_ABS, ABS_MISC);
    input_set_abs_params(input, ABS_X, 0, 1, 0, 0);
    input_set_drvdata(input, NULL);
    err = input_register_device(input);


    input2 = input_allocate_device();
    input2->name = "light";
    input2->id.bustype = BUS_I2C;

    input_set_capability(input2, EV_ABS, ABS_MISC);
    input_set_abs_params(input2, ABS_X, 0, 1, 0, 0);
    input_set_drvdata(input2, NULL);
    err = input_register_device(input2);

    INIT_DELAYED_WORK(&gp2_light_work,GP2AP002A00F_poll_light_sensor);


	

    atomic_set(&proximity_data_tbl.enable, 0);
    atomic_set(&proximity_data_tbl.delay, msecs_to_jiffies(10));
    proximity_data_tbl.last_data = 0;

    err = sysfs_create_group(&input->dev.kobj,&proximity_attribute_group);

    atomic_set(&light_data_tbl.enable, 0);
    atomic_set(&light_data_tbl.delay, msecs_to_jiffies(LIGHT_SENSOR_POLL_TIME_MSEC));
    light_data_tbl.last_data = 0;

    err = light_create_group(input2);

    
    HS_A_PLOXIMITY_ON_LO();
    HS_A_PHOTO_ON_LO();
    


    printk("[GP2AP002A00F] END:GP2AP002A00F_init_input\n");

	return 0;
}


static int GP2AP002A00F_open(struct inode *inode, struct file *file)
{
	return nonseekable_open(inode, file);
}

static int GP2AP002A00F_release(struct inode *inode, struct file *file)
{
	return 0;
}
static int GP2AP002A00F_ioctl(struct inode *inode, struct file *file,
				unsigned int cmd, unsigned long arg)
{
	switch(cmd)
	{
		case GP2_IOCTL_CMD:
			{
				unsigned long ret = 0;
				s32 param[2] = {0};
				int new_mode = 0;
				
				ret = copy_from_user(&param,(int __user *)arg,sizeof(param));
				new_mode = param[0];
				if(new_mode != gp2_mode)
				{
					printk("@@@DEBUG GP2AP002A00F_ioctl GP2_IOCTL_CMD new_mode[0x%x]\n",param[0]);
					gp2_mode_chg_req = new_mode;
				}
				if(new_mode == GP2_MODE_CUSTOM)
				{
					memcpy(&reg_set_list[new_mode],&reg_set_list[gp2_mode],sizeof(reg_set_list[new_mode]));
					reg_set_list[GP2_MODE_CUSTOM].hys.hysd = param[1]&0x80;
					reg_set_list[GP2_MODE_CUSTOM].hys.hysc = param[1]&0x60;
					reg_set_list[GP2_MODE_CUSTOM].hys.hysf = param[1]&0x0F;
				}
			}
			break;
		default:
			break;
	}
	return 0;
}

static struct file_operations GP2AP002A00F_fops = {
	.owner = THIS_MODULE,
	.open  = GP2AP002A00F_open,
	.release = GP2AP002A00F_release,
	.ioctl   = GP2AP002A00F_ioctl,
};

static struct miscdevice GP2AP002A00F_device = {
	.minor = MISC_DYNAMIC_MINOR,
	.name  = PROXIMITY_DEV_NAME,
	.fops = &GP2AP002A00F_fops,
};



static int
GP2AP002A00F_probe(struct i2c_client *client, const struct i2c_device_id *id)
{

    printk("[GP2AP002A00F] START:GP2AP002A00F_probe\n");


#if defined(CONFIG_TARGET_PROTOTYPE_WS2) | defined(CONFIG_TARGET_PROTOTYPE_WS3)
    
	HS_A_PLOXIMITY_ON_HI();
	HS_A_PHOTO_ON_LO();
    
	gpio_tlmm_config(GPIO_CFG(GPIO_PLOXIMITY_ON, 0, GPIO_CFG_OUTPUT, GPIO_CFG_NO_PULL, GPIO_CFG_2MA), GPIO_CFG_ENABLE);
	gpio_tlmm_config(GPIO_CFG(GPIO_PHOTO_ON, 0, GPIO_CFG_OUTPUT, GPIO_CFG_NO_PULL, GPIO_CFG_2MA), GPIO_CFG_ENABLE);

    
    gpio_photo_on_value = GPIO_LO;

#endif 
	gpio_tlmm_config(GPIO_CFG(GPIO_PS_OUT, 0, GPIO_CFG_INPUT, GPIO_CFG_PULL_DOWN, GPIO_CFG_2MA), GPIO_CFG_ENABLE);
	
	
    i2c_set_clientdata(client, NULL);
	this_client = client;

	if(misc_register(&GP2AP002A00F_device))
	{

		printk("[GP2AP002A00F] ERR:misc_register error\n");

	}
	

	GP2AP002A00F_init_input();


	wake_lock_init(&proximity_wake_lock, WAKE_LOCK_SUSPEND, "proximity");  


    printk("[GP2AP002A00F] END:GP2AP002A00F_probe\n");

    return 0;
}


static int
GP2AP002A00F_remove(struct i2c_client *client)
{


    SENSOR_DPRINTK("START:GP2AP002A00F_remove\n");

    proximity_disable();
    light_disable();

    sysfs_remove_group(&input->dev.kobj, &proximity_attribute_group);

    input_unregister_device(input);
    input_free_device(input);

    SENSOR_DPRINTK("END:GP2AP002A00F_remove\n");


    return 0;
}

static int
GP2AP002A00F_suspend(struct i2c_client *client, pm_message_t mesg)
{

    SENSOR_DPRINTK("START:GP2AP002A00F_suspend\n");



    SENSOR_DPRINTK("END:GP2AP002A00F_suspend\n");

    return 0;
}

static int
GP2AP002A00F_resume(struct i2c_client *client)
{

    SENSOR_DPRINTK("START:GP2AP002A00F_resume\n");



    SENSOR_DPRINTK("END:GP2AP002A00F_resume\n");

    return 0;
}


static struct i2c_device_id GP2AP002A00F_idtable[] = {
    {PROXIMITY_DEV_NAME, 0},
    {}
};
MODULE_DEVICE_TABLE(i2c, GP2AP002A00F_idtable);

static struct i2c_driver GP2AP002A00F_i2c_driver = {
    .driver = {
        .name       = PROXIMITY_DEV_NAME,
        .owner      = THIS_MODULE,
    },

    .id_table       = GP2AP002A00F_idtable,
    .probe          = GP2AP002A00F_probe,
    .remove         = GP2AP002A00F_remove,
    .suspend        = GP2AP002A00F_suspend,
    .resume         = GP2AP002A00F_resume,
};

static int __init
GP2AP002A00F_init(void)
{
    return i2c_add_driver(&GP2AP002A00F_i2c_driver);
}

static void __exit
GP2AP002A00F_term(void)
{
    i2c_del_driver(&GP2AP002A00F_i2c_driver);
}

module_init(GP2AP002A00F_init);
module_exit(GP2AP002A00F_term);

MODULE_AUTHOR("Kyocera Corporation");
MODULE_DESCRIPTION("Kyocera proximity Sensor Driver");
MODULE_LICENSE( "GPL" );
MODULE_VERSION("1.0.0");
