/* This software is contributed or developed by KYOCERA Corporation.
 * (C) 2011 KYOCERA Corporation
 */











#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/platform_device.h>
#include <linux/leds.h>
#include <linux/fs.h>
#include <linux/syscalls.h>
#include <linux/unistd.h>
#include <linux/fcntl.h>

#include <mach/pmic.h>
#include <mach/../../proc_comm.h>

struct light_state_t {
    unsigned int color;
    int flashMode;
    int flashOnMS;
    int flashOffMS;
    int brightnessMode;
};

static struct light_state_t led_data[5];
static struct timer_list    timer_led;
static struct workqueue_struct *led_work;
static struct work_struct   work_led;
static struct workqueue_struct *ledstate_work;
static struct work_struct   work_ledstate;
static int timer_state;
static int led_on_off;
static int led_value;
static int fs_state;
static int led_state;

static void led_msm_pmic_red_brightness(struct led_classdev *led_cdev,
                                          enum led_brightness value)
{
    printk("led RED brightness %d\n",value);
    if(value){
        pmic_secure_mpp_config_i_sink(PM_MPP_12,
                         PM_MPP__I_SINK__LEVEL_5mA,
                         PM_MPP__I_SINK__SWITCH_ENA);
    }
    else{
        pmic_secure_mpp_config_i_sink(PM_MPP_12,
                         PM_MPP__I_SINK__LEVEL_5mA,
                         PM_MPP__I_SINK__SWITCH_DIS);
    }
}
static void led_msm_pmic_green_brightness(struct led_classdev *led_cdev,
	enum led_brightness value)
{
    printk("led GREEN brightness %d\n",value);
    if(value){
        pmic_secure_mpp_config_i_sink(PM_MPP_11,
                         PM_MPP__I_SINK__LEVEL_5mA,
                         PM_MPP__I_SINK__SWITCH_ENA);
    }
    else{
        pmic_secure_mpp_config_i_sink(PM_MPP_11,
                         PM_MPP__I_SINK__LEVEL_5mA,
                         PM_MPP__I_SINK__SWITCH_DIS);
    }
}
static void led_msm_pmic_blue_brightness(struct led_classdev *led_cdev,
	enum led_brightness value)
{
    printk("led BLUE brightness %d\n",value);

}

static void led_msm_pmic_state_brightness(struct led_classdev *led_cdev,
	enum led_brightness value)
{
    printk("led state brightness %d\n",value);

    if(value > 0 && 6 > value){
        led_value = value;

        if(fs_state)queue_work(ledstate_work, &work_ledstate);
    }
}

static void ledstate_work_0(struct work_struct *work)
{

  struct file *filp;
  struct light_state_t  buf;
  int nr_read;
  int data1,data2;
  int color;

  filp = filp_open("/dev/ledstate", O_RDONLY, 0);
  if (!filp) {
          return;
  }


  nr_read = filp->f_op->read(filp, (char *)&buf, sizeof(struct light_state_t), &filp->f_pos);
  printk("********* led state start\n");
  printk("color : 0x%x\n", buf.color);
  printk("flashMode : %d\n", buf.flashMode);
  printk("flashOnMS : %d\n", buf.flashOnMS);
  printk("flashOffMS : %d\n", buf.flashOffMS);
  printk("brightnessMode : %d\n", buf.brightnessMode);
  printk("********* led state end\n");

  filp_close(filp, NULL);


  data1=8;
  color =  (buf.color&0x000000ff)?1:0;
  color += (buf.color&0x0000ff00)?2:0;
  color += (buf.color&0x00ff0000)?4:0;
  data2=((led_value-1)<<24)|(color<<16)|(buf.flashMode<<8)|(buf.brightnessMode);
  printk("proc_comm data2 : 0x%08x\n",data2);
  msm_proc_comm(PCOM_CUSTOMER_CMD2, &data1, &data2);

  data1=9;
  data2=(buf.flashOnMS<<16)|(buf.flashOffMS);
  printk("proc_comm data2 : 0x%08x\n",data2);
  msm_proc_comm(PCOM_CUSTOMER_CMD2, &data1, &data2);

  if(fs_state)queue_work(led_work, &work_led);

    return;

}

static void led_work_0(struct work_struct *work)
{

  struct light_state_t  buf;
  int fd;

  if(!fs_state){
    if((fd = sys_open("/dev/ledstate", O_RDWR|O_CREAT, 0666)) != -1) {
        if (sys_ftruncate(fd, 256) != -1) {
            sys_write(fd, (void*)&buf, sizeof(struct light_state_t));
            sys_close(fd);
            fs_state = 1;
            return;
        }
    }
    mod_timer(&timer_led,jiffies + msecs_to_jiffies(10));
    return;
  }

}

static void led_timer_start(unsigned long data)
{
  queue_work(led_work, &work_led);
}


static struct led_classdev led_msm_pmic_red = {
	.name			= "red",
	.brightness_set		= led_msm_pmic_red_brightness,
	.brightness		= LED_OFF,
};
static struct led_classdev led_msm_pmic_green = {
	.name			= "green",
	.brightness_set		= led_msm_pmic_green_brightness,
	.brightness		= LED_OFF,
};
static struct led_classdev led_msm_pmic_blue = {
	.name			= "blue",
	.brightness_set		= led_msm_pmic_blue_brightness,
	.brightness		= LED_OFF,
};

static struct led_classdev led_msm_pmic_state = {
	.name			= "state",
	.brightness_set		= led_msm_pmic_state_brightness,
	.brightness		= LED_OFF,
};

static int led_msm_pmic_rgb_probe(struct platform_device *pdev)
{
    int rc;

    rc = led_classdev_register(&pdev->dev, &led_msm_pmic_red);
    if (rc) {
        dev_err(&pdev->dev, "unable to register led class driver\n");
        return rc;
    }
    rc = led_classdev_register(&pdev->dev, &led_msm_pmic_green);
    if (rc) {
        dev_err(&pdev->dev, "unable to register led class driver\n");
        return rc;
    }
    rc = led_classdev_register(&pdev->dev, &led_msm_pmic_blue);
    if (rc) {
        dev_err(&pdev->dev, "unable to register led class driver\n");
        return rc;
    }

    rc = led_classdev_register(&pdev->dev, &led_msm_pmic_state);
    if (rc) {
        dev_err(&pdev->dev, "unable to register led class driver\n");
        return rc;
    }


    INIT_WORK(&work_led, led_work_0);
    INIT_WORK(&work_ledstate, ledstate_work_0);
    setup_timer(&timer_led, led_timer_start,0);

    mod_timer(&timer_led,jiffies + msecs_to_jiffies(1000));

    return rc;
}

static int __devexit led_msm_pmic_rgb_remove(struct platform_device *pdev)
{
	led_classdev_unregister(&led_msm_pmic_red);
	led_classdev_unregister(&led_msm_pmic_green);
	led_classdev_unregister(&led_msm_pmic_blue);
	led_classdev_unregister(&led_msm_pmic_state);

	return 0;
}

#ifdef CONFIG_PM
static int led_msm_pmic_rgb_suspend(struct platform_device *dev,
		pm_message_t state)
{
	return 0;
}

static int led_msm_pmic_rgb_resume(struct platform_device *dev)
{
	return 0;
}
#else
#define led_msm_pmic_rgb_suspend NULL
#define led_msm_pmic_rgb_resume NULL
#endif

static struct platform_driver led_msm_pmic_rgb_driver = {
	.probe		= led_msm_pmic_rgb_probe,
	.remove		= __devexit_p(led_msm_pmic_rgb_remove),
	.suspend	= led_msm_pmic_rgb_suspend,
	.resume		= led_msm_pmic_rgb_resume,
	.driver		= {
		.name	= "pmic-leds-rgb",
		.owner	= THIS_MODULE,
	},
};

static int __init led_msm_pmic_rgb_init(void)
{
    timer_state = 0;
    led_on_off = 0;
    fs_state = 0;
    led_state = 0;
    memset(led_data,0,sizeof(led_data));
    led_work = create_singlethread_workqueue("led_work");
    ledstate_work = create_singlethread_workqueue("ledstate_work");
    return platform_driver_register(&led_msm_pmic_rgb_driver);
}
module_init(led_msm_pmic_rgb_init);

static void __exit led_msm_pmic_rgb_exit(void)
{
    if (led_work)
        destroy_workqueue(led_work);
    platform_driver_unregister(&led_msm_pmic_rgb_driver);
}
module_exit(led_msm_pmic_rgb_exit);

MODULE_DESCRIPTION("MSM PMIC LEDs driver");
MODULE_LICENSE("GPL v2");
MODULE_ALIAS("platform:pmic-leds-rgb");

