/* This software is contributed or developed by KYOCERA Corporation. */
/* (C) 2011 KYOCERA Corporation                                      */

#ifdef CONFIG_KYOCERA_ORIGINAL_FEATURE
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/time.h>
#include <linux/init.h>
#include <linux/interrupt.h>
#include <linux/hrtimer.h>

#include <mach/hardware.h>
#include <linux/io.h>

#include <linux/font.h>

#include <asm/system.h>
#include <asm/mach-types.h>
#include <linux/semaphore.h>

#include <linux/fb.h>

#include "../console/fbcon.h"

#include "msm_fb.h"
#if DTS_LCD_FBPRINT

boolean fbprint_update_flg = false;

#define FBPRINT_RECT_W (480)
#define FBPRINT_RECT_H (800)

static int fbprint_rect_x = 0;
static int fbprint_rect_y = 0;
static int fbprint_rect_w = 0;
static int fbprint_rect_h = 0;

static uint16 *fbprint_buf[2] = {NULL,NULL};

static uint16 *fbprint_font_dat = NULL;

static int fbprint_font_w = 0;
static int fbprint_font_h = 0;

#if 1
#define LOCAL_FUNC static
#else
#define LOCAL_FUNC
#endif

LOCAL_FUNC void fbprint_init(void);

void fbprint(
    char *format, 
    ...           
                  
) {
    char str[DTS_LCD_FBPRINT_MAX_LENGTH];
    char *strp = str;
    va_list arg;
    int x, y;
    uint16 *p_y, *p_x, *s, *d;
    char c;

    if (fbprint_buf[0] == NULL) {
        fbprint_init();
    }
    fbprint_rect_x = 0;
    fbprint_rect_y = 0;
    fbprint_rect_w = 0;
    fbprint_rect_h = 0;

    if (format != NULL) {
        va_start(arg, format);
        vsprintf(str, format, arg);
        va_end(arg);

        x = 0;
        p_x = p_y = fbprint_buf[0];

        for (c=*strp; c!='\0'; c=*(++strp)) {
            if (x == 0) {
                fbprint_rect_h += fbprint_font_h;
                if (fbprint_rect_h > FBPRINT_RECT_H) {
                    fbprint_rect_h = FBPRINT_RECT_H;
                    break;
                }
                memset(p_y, 0x00, (FBPRINT_RECT_W * 2) * fbprint_font_h);
            }

            if (c != '\n') {
                s = fbprint_font_dat + (uint8)c * fbprint_font_w * fbprint_font_h;
                d = p_x;
                for (y=0; y<fbprint_font_h; y++) {
                    memcpy(d, s, fbprint_font_w * 2);
                    s += fbprint_font_w;
                    d += FBPRINT_RECT_W;
                }

                x += fbprint_font_w;
                if (fbprint_rect_w < x) {
                    fbprint_rect_w = x;
                }
                p_x += fbprint_font_w;
            }

            if (c == '\n' || x >= FBPRINT_RECT_W) {
                x = 0;
                p_y += FBPRINT_RECT_W * fbprint_font_h;
            }
        }
    }

    fbprint_update_flg = true;
}

int mdp_chk_fbprint(struct fb_info *info, struct mdp_dirty_region *dirtyPtr)
{
    int ib_x2, ib_y2, fb_x2, fb_y2;

    if (fbprint_rect_w == 0 || fbprint_rect_h == 0 || fbprint_buf[0] == NULL) {
        return 0;
    }

    ib_x2 = dirtyPtr->xoffset + dirtyPtr->width;
    ib_y2 = dirtyPtr->yoffset + dirtyPtr->height;
    fb_x2 = fbprint_rect_x + fbprint_rect_w;
    fb_y2 = fbprint_rect_y + fbprint_rect_h;

    dirtyPtr->xoffset = (dirtyPtr->xoffset < fbprint_rect_x ? dirtyPtr->xoffset : fbprint_rect_x);
    dirtyPtr->yoffset = (dirtyPtr->yoffset < fbprint_rect_y ? dirtyPtr->yoffset : fbprint_rect_y);
    dirtyPtr->width   = (ib_x2 > fb_x2 ? ib_x2 : fb_x2) - dirtyPtr->xoffset;
    dirtyPtr->height  = (ib_y2 > fb_y2 ? ib_y2 : fb_y2) - dirtyPtr->yoffset;

    return 1;
}

void mdp_ppp_fbprint(struct fb_info *info)
{
    extern struct semaphore mdp_ppp_mutex;

    struct msm_fb_data_type *mfd = (struct msm_fb_data_type *)info->par;
    MDPIBUF *iBuf = &mfd->ibuf_tbl[mfd->ibuf_idx_ppp_vsync];
    uint32 src_x, src_y, src_w, src_h, src_b;
    uint32 dst_x, dst_y, dst_w, dst_h, dst_b;
    uint32 src0_ystride;
    uint32 dst0_ystride;
    uint32 src_cfg, dst_cfg, ppp_op;
    uint32 src_pat, dst_pat;
    uint8 *src0, *src1;
    uint8 *dst0, *dst1;
    int sv_slice, sh_slice;
    int dv_slice, dh_slice;

    fbprint_update_flg = false;

    if (fbprint_rect_w == 0 || fbprint_rect_h == 0 || fbprint_buf[0] == NULL) {
        return;
    }

    down(&mdp_ppp_mutex);
    mdp_pipe_ctrl(MDP_CMD_BLOCK, MDP_BLOCK_POWER_ON, FALSE);

    src_x = dst_x = 0;
    src_y = dst_y = 0;
    src_w = dst_w = iBuf->ibuf_width;
    src_h = dst_h = iBuf->ibuf_height;
    src_b = dst_b = iBuf->bpp;
    src1  = dst1  = NULL;
    sv_slice = sh_slice = dv_slice = dh_slice = 1;

    do { } while (mdp_ppp_pipe_wait() <= 0);

    src_pat = MDP_GET_PACK_PATTERN(0, CLR_R, CLR_G, CLR_B, 8);
    src_cfg =
        PPP_SRC_C0G_6BITS |
        PPP_SRC_C1B_5BITS |
        PPP_SRC_C2R_5BITS |
        PPP_SRC_BPP_INTERLVD_2BYTES |
        PPP_SRC_INTERLVD_3COMPONENTS |
        PPP_SRC_UNPACK_TIGHT |
        PPP_SRC_UNPACK_ALIGN_LSB |
        PPP_SRC_FETCH_PLANES_INTERLVD;

    switch (iBuf->ibuf_type) {
    case MDP_RGB_888:
        dst_pat = MDP_GET_PACK_PATTERN(0, CLR_R, CLR_G, CLR_B, 8);
        dst_cfg =
            PPP_DST_C0G_8BIT |
            PPP_DST_C1B_8BIT |
            PPP_DST_C2R_8BIT |
            PPP_DST_BPP_3BYTES |
            PPP_DST_PACKET_CNT_INTERLVD_3ELEM |
            PPP_DST_PACK_TIGHT |
            PPP_DST_PACK_ALIGN_LSB |
            PPP_DST_OUT_SEL_AXI |
            PPP_DST_PLANE_INTERLVD;
        break;
    case MDP_BGR_565:
    case MDP_RGB_565:
    default:
        if (iBuf->ibuf_type == MDP_RGB_565 ){
            dst_pat = MDP_GET_PACK_PATTERN(0, CLR_R, CLR_G, CLR_B, 8);
        } else {
            dst_pat = MDP_GET_PACK_PATTERN(0, CLR_B, CLR_G, CLR_R, 8);
        }
        dst_cfg =
            PPP_DST_C0G_6BIT |
            PPP_DST_C1B_5BIT |
            PPP_DST_C2R_5BIT |
            PPP_DST_BPP_2BYTES |
            PPP_DST_PACKET_CNT_INTERLVD_3ELEM |
            PPP_DST_PACK_TIGHT |
            PPP_DST_PACK_ALIGN_LSB |
            PPP_DST_OUT_SEL_AXI |
            PPP_DST_PLANE_INTERLVD;
        break;
    }

    ppp_op =
        PPP_OP_COLOR_SPACE_RGB |
        PPP_OP_SRC_CHROMA_RGB |
        PPP_OP_DST_CHROMA_RGB;

    src0_ystride = src_w * src_b;
    dst0_ystride = dst_w * dst_b;

    src0 = (uint8 *)fbprint_buf[1] +
        (fbprint_rect_x + fbprint_rect_y * FBPRINT_RECT_W) * 2;
    dst0 = (uint8 *)iBuf->buf +
        (fbprint_rect_x + fbprint_rect_y * FBPRINT_RECT_W) * 2;

    mdp_adjust_start_addr(&src0, &src1, sv_slice, sh_slice,
        src_x, src_y, src_w, src_h, src_b, iBuf, 0);
    mdp_adjust_start_addr(&dst0, &dst1, dv_slice, dh_slice,
        dst_x, dst_y, dst_w, dst_h, dst_b, iBuf, 2);

    MDP_OUTP(MDP_PPP_SRC_SIZE,            (fbprint_rect_h << 16 | fbprint_rect_w));
    MDP_OUTP(MDP_PPP_SRCP0_ADDR,          src0);
    MDP_OUTP(MDP_PPP_SRCP1_ADDR,          src1);
    MDP_OUTP(MDP_PPP_SRC_YSTRIDE1,        (src0_ystride << 16 | src0_ystride));

    MDP_OUTP(MDP_PPP_SRC_FORMAT,          src_cfg);
    MDP_OUTP(MDP_PPP_SRC_UNPACK_PATTERN1, src_pat);

    MDP_OUTP(MDP_PPP_OP_MODE,             ppp_op);
    MDP_OUTP(MDP_PPP_BLEND_PARAM,         0);

    MDP_OUTP(MDP_PPP_OUT_FORMAT,          dst_cfg);
    MDP_OUTP(MDP_PPP_OUT_PACK_PATTERN1,   dst_pat);

    MDP_OUTP(MDP_PPP_OUT_SIZE,            (fbprint_rect_h << 16 | fbprint_rect_w));
    MDP_OUTP(MDP_PPP_OUTP0_ADDR,          dst0);
    MDP_OUTP(MDP_PPP_OUTP1_ADDR,          dst1);
    MDP_OUTP(MDP_PPP_OUT_YSTRIDE1,        (dst0_ystride << 16 | dst0_ystride));

    mdp_pipe_kickoff(MDP_PPP_TERM, mfd, FALSE);

    mdp_pipe_ctrl(MDP_CMD_BLOCK, MDP_BLOCK_POWER_OFF, FALSE);
    up(&mdp_ppp_mutex);

}

void mdp_dma_fbprint(struct fb_info *info)
{
    struct msm_fb_data_type *mfd = (struct msm_fb_data_type *)info->par;

    fbprint_update_flg = false;

    if (fbprint_rect_w == 0 || fbprint_rect_h == 0 || fbprint_buf[0] == NULL) {
        return;
    }

	if (!mfd->panel_power_on) {
        return;
    }

    down(&mfd->sem);
    {
        mdp_enable_irq(MDP_DMA_S_TERM);
        mfd->dma->busy = TRUE;
        INIT_COMPLETION(mfd->dma->comp);

        mdp_pipe_ctrl(MDP_CMD_BLOCK, MDP_BLOCK_POWER_ON, TRUE);

        MDP_OUTP(MDP_BASE + 0xa0004, (fbprint_rect_h << 16) | fbprint_rect_w);
        MDP_OUTP(MDP_BASE + 0xa0008,
            (uint8 *)fbprint_buf[1] + (fbprint_rect_x + fbprint_rect_y * FBPRINT_RECT_W) * 2);
        MDP_OUTP(MDP_BASE + 0xa000c, FBPRINT_RECT_W * 2); /* ystride */
        MDP_OUTP(MDP_BASE + 0xa0010, (fbprint_rect_y << 16) | fbprint_rect_x);
        MDP_OUTP(MDP_BASE + 0x00090, 1);
        MDP_OUTP(MDP_BASE + 0x00094, (MDDI_VDO_PACKET_DESC << 16) | mfd->panel_info.mddi.vdopkt);
        MDP_OUTP(MDP_BASE + 0xa0000,
            DMA_PACK_TIGHT |
            DMA_PACK_ALIGN_LSB |
            DMA_OUT_SEL_AHB |
            DMA_IBUF_NONCONTIGUOUS |
            DMA_PACK_PATTERN_RGB |
            DMA_IBUF_FORMAT_RGB565 |
            DMA_OUT_SEL_MDDI |
            DMA_DITHER_EN |
            DMA_DSTC0G_6BITS |
            DMA_DSTC1B_6BITS |
            DMA_DSTC2R_6BITS);

        mdp_pipe_ctrl(MDP_CMD_BLOCK, MDP_BLOCK_POWER_OFF, TRUE);

        mdp_pipe_kickoff(MDP_DMA_S_TERM, mfd, TRUE);
    }
    up(&mfd->sem);
}

LOCAL_FUNC void fbprint_init(void)
{
    int size;
    int n, x, y, c;
    uint8 *s;
    uint16 *d;

    size = (FBPRINT_RECT_W * 2) * FBPRINT_RECT_H;
    fbprint_buf[0] = (uint16 *)kmalloc(size, GFP_ATOMIC | GFP_DMA);
    fbprint_buf[1] = (uint16 *)virt_to_phys(fbprint_buf[0]);

    memset(fbprint_buf[0], 0x00, size);

    n = 256;
    fbprint_font_w = font_vga_8x16.width;
    fbprint_font_h = font_vga_8x16.height;
    s = (uint8 *)font_vga_8x16.data;

    size = n * (fbprint_font_w * 2) * fbprint_font_h;
    fbprint_font_dat = (uint16 *)kmalloc(size, GFP_ATOMIC | GFP_DMA);

    d = fbprint_font_dat;
    for (c=0; c<n; c++) {
        for (y=0; y<fbprint_font_h; y++) {
            for (x=0; x<fbprint_font_w; x++) {
                if (*s & (0x80 >> x)) {
                    *(d++) = 0xFFFF;
                } else {
                    *(d++) = 0x0000;
                }
            }
            s++;
        }   
    }
}
#endif 
#endif 
